diff a/src/hotspot/share/oops/instanceKlassFlags.hpp b/src/hotspot/share/oops/instanceKlassFlags.hpp
--- a/src/hotspot/share/oops/instanceKlassFlags.hpp
+++ b/src/hotspot/share/oops/instanceKlassFlags.hpp
@@ -53,12 +53,23 @@
     flag(has_contended_annotations          , 1 << 10) /* has @Contended annotation */ \
     flag(has_localvariable_table            , 1 << 11) /* has localvariable information */ \
     flag(has_miranda_methods                , 1 << 12) /* True if this class has miranda methods in it's vtable */ \
     flag(has_vanilla_constructor            , 1 << 13) /* True if klass has a vanilla default constructor */ \
     flag(has_final_method                   , 1 << 14) /* True if klass has final method */ \
+    flag(has_inline_type_fields             , 1 << 15) /* has inline fields and related embedded section is not empty */ \
+    flag(is_empty_inline_type               , 1 << 16) /* empty inline type (*) */ \
+    flag(is_naturally_atomic                , 1 << 17) /* loaded/stored in one instruction */ \
+    flag(must_be_atomic                     , 1 << 18) /* doesn't allow tearing */ \
+    flag(has_loosely_consistent_annotation  , 1 << 19) /* the class has the LooselyConsistentValue annotation WARNING: it doesn't automatically mean that the class allows tearing */ \
+    flag(is_implicitly_constructible        , 1 << 20) /* the class has the ImplicitlyConstrutible annotation */
     /* end of list */
 
+  /* (*) An inline type is considered empty if it contains no non-static fields or
+     if it contains only empty inline fields. Note that JITs have a slightly different
+     definition: empty inline fields must be flat otherwise the container won't
+     be considered empty */
+
 #define IK_FLAGS_ENUM_NAME(name, value)    _misc_##name = value,
   enum {
     IK_FLAGS_DO(IK_FLAGS_ENUM_NAME)
   };
 #undef IK_FLAGS_ENUM_NAME
@@ -80,11 +91,11 @@
   u2 shared_loader_type_bits() const {
     return _misc_is_shared_boot_class|_misc_is_shared_platform_class|_misc_is_shared_app_class;
   }
 
   // These flags are write-once before the class is published and then read-only so don't require atomic updates.
-  u2 _flags;
+  u4 _flags;
 
   // These flags are written during execution so require atomic stores
   u1 _status;
 
  public:
@@ -106,10 +117,17 @@
   }
 
   void set_shared_class_loader_type(s2 loader_type);
 
   void assign_class_loader_type(const ClassLoaderData* cld);
+
+  u4 flags() const { return _flags; }
+
+  static u4 is_empty_inline_type_value() {
+    return _misc_is_empty_inline_type;
+  }
+
   void assert_is_safe(bool set) NOT_DEBUG_RETURN;
 
   // Create getters and setters for the status values.
 #define IK_STATUS_GET_SET(name, ignore)          \
   bool name() const { return (_status & _misc_##name) != 0; } \
